<?php

namespace Bixo\Car\Http\Controllers;

use Exception;
use Bixo\Car\Actions\CarWorkflow;
use Bixo\Car\Http\Requests\CarWorkflowRequest;
use Bixo\Car\Http\Resources\CarResource;
use Bixo\Car\Models\Car;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class CarWorkflowController extends BaseController
{
    /**
     * Action controller function for car.
     *
     * @param Model $car
     * @param action next action for the car.
     *
     * @return Response
     */
    public function __invoke(CarWorkflowRequest $request, Car $car, $trasition)
    {
        try {
            $request = $request->all();
            $car = CarWorkflow::run($trasition, $car, $request);
            $data = new CarResource($car);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('car::car.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('car/car/' . $car->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('car/car/' . $car->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the car and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $car
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $car, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $car = Car::findBySignedId($car);
        $transitions = $this->transitions($car, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('car::car.name') . ' ' . trans('Approvals'))
            ->view('car::public.car.workflow')
            ->layout('mobile')
            ->data(compact('car', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the car.
     *
     * @param PublicRequest $request
     * @param string $car
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $car, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $car = Car::findBySignedId($car);
        $user = user();
        $car = CarWorkflow::run($transition, $car, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($car, $user)
    {
        $transitions = [];
        foreach ($car->workflow()->transitions($car) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.car.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'car' => $car->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $car->workflow()->form($value);
            $array['label'] = trans('car::car.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
